@@ -28,6 +28,8 @@ import itertools
 from cli.messages import Messages
 import types
 import re
+import keyword
+import __builtin__
 
 
 class OvirtCommand(Command):
@@ -96,7 +98,7 @@ class OvirtCommand(Command):
             return param
 
     def __do_set_primitive_list_data(self, obj, prop, val):
-        for param in str(val).split(','):
+        for param in self.__split_with_escape(str(val), delimiter=','):
             getattr(obj, prop).append(param)
 
     def __do_set_data(self, obj, prop, fq_prop, val):
@@ -121,7 +123,7 @@ class OvirtCommand(Command):
                         if not val:
                             self.error(Messages.Error.INVALID_COLLECTION_BASED_OPTION_SYNTAX % prop)
 
-                        for param in str(val).split(','):
+                        for param in self.__split_with_escape(str(val), delimiter=','):
                             obj_params_set_cand = root_obj_params_set_cand
                             if not param.startswith(props[i] + '.'):
                                 self.error(
@@ -133,6 +135,8 @@ class OvirtCommand(Command):
                                 spplited_param_data = param_data[0].split('.')
                                 for param_period in spplited_param_data:
                                     if spplited_param_data[-1] == param_period:
+                                        if not hasattr(obj_params_set_cand, param_period):
+                                            param_period = self.fixParamNameIfParamIsKeyword(param_period)
                                         if hasattr(obj_params_set_cand, param_period):
                                             if getattr(obj_params_set_cand, param_period) != None:
                                                 getattr(obj, props[i]).append(obj_params_set_cand)
@@ -187,6 +191,11 @@ class OvirtCommand(Command):
         else:
             self.__set_property(obj, prop, val, fq_prop)
 
+    def fixParamNameIfParamIsKeyword(self, param):
+        if param in dir(__builtin__) or keyword.iskeyword(param):
+            return param + '_'
+        return param
+
     def __set_property(self, obj, prop, val, fq_prop):
         """INTERNAL: set data in to property"""
         if hasattr(obj, prop):
@@ -336,15 +345,16 @@ class OvirtCommand(Command):
             coll = getattr(base, candidate + 's')
             if coll is not None:
                 if name:
-                    return coll.get(name=name)
+                    return self.__get_by_name(coll, name, kwargs)
                 if kwargs and kwargs.has_key('id'):
-                    return coll.get(id=str(kwargs['id']))
+                    id, kwargs = self._get_query_params(opts, query_arg='--id')
+                    return self.__get_by_id(coll, str(id), kwargs)
                 else:
                     identifier = self.__produce_identifier(obj_id)
                     if identifier:
-                        return coll.get(id=str(obj_id))
+                        return self.__get_by_id(coll, str(obj_id), kwargs)
                     else:
-                        return coll.get(name=obj_id)
+                        return self.__get_by_name(coll, obj_id, kwargs)
         else:
             err_str = Messages.Error.NO_SUCH_TYPE_OR_ARS_NOT_VALID
             if context_variants:
@@ -354,6 +364,18 @@ class OvirtCommand(Command):
 
         return None
 
+    def __get_by_name(self, coll, name, kwargs):
+        if kwargs:
+            return coll.get(name=name, **kwargs)
+        else:
+            return coll.get(name=name)
+
+    def __get_by_id(self, coll, id, kwargs):
+        if kwargs:
+            return coll.get(id=id, **kwargs)
+        else:
+            return coll.get(id=id)
+
     def __produce_identifier(self, candidate):
         if type(candidate) == str:
             match = re.search(r'[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}', candidate)
@@ -486,3 +508,45 @@ class OvirtCommand(Command):
         """INTERNAL: return a list of type actions."""
 
         return MethodHelper.get_object_methods(obj, exceptions=['delete', 'update'])
+
+    def __split_with_escape(self, text, delimiter=',', escape='\\'):
+        """
+        Splits a string so that the delimiter is ignored if it is preceded by
+        the escape character.
+        """
+
+        chunks = []
+        chunk = ''
+        escaping = False
+        for c in list(text) + [None]:
+            if not escaping:
+                if c is None:
+                    chunks.append(chunk)
+                    chunk = ''
+                    escaping = False
+                elif c == escape:
+                    escaping = True
+                elif c == delimiter:
+                    chunks.append(chunk)
+                    chunk = ''
+                    escaping = False
+                else:
+                    chunk += c
+                    escaping = False
+            else:
+                if c is None:
+                    chunk += escape
+                    chunks.append(chunk)
+                    chunk = ''
+                    escaping = False
+                elif c == escape:
+                    chunk += escape
+                    escaping = False
+                elif c == delimiter:
+                    chunk += c
+                    escaping = False
+                else:
+                    chunk += escape
+                    chunk += c
+                    escaping = False
+        return chunks
